from enum import Enum


class CommandListItemCommand(str, Enum):
    CLIMATIZATION_START = "CLIMATIZATION_START"
    CLIMATIZATION_STOP = "CLIMATIZATION_STOP"
    ENGINE_START = "ENGINE_START"
    ENGINE_STOP = "ENGINE_STOP"
    FLASH = "FLASH"
    HONK = "HONK"
    HONK_AND_FLASH = "HONK_AND_FLASH"
    LOCK = "LOCK"
    LOCK_REDUCED_GUARD = "LOCK_REDUCED_GUARD"
    SEND_NAVI_POI = "SEND_NAVI_POI"
    UNLOCK = "UNLOCK"

    def __str__(self) -> str:
        return str(self.value)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------

project = u"connected_vehicle_client"
copyright = u"2023, exbachi"
author = u"exbachi"

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "myst_nb",
    "autoapi.extension",
    "sphinx.ext.napoleon",
    "sphinx.ext.viewcode",
]
autoapi_dirs = ["../src"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.r_instance_with_unit import RInstanceWithUnit


T = TypeVar("T", bound="StatisticVals")


@attr.s(auto_attribs=True)
class StatisticVals:
    """
    Attributes:
        average_speed (Union[Unset, RInstanceWithUnit]):
        distance_to_empty (Union[Unset, RInstanceWithUnit]):
        trip_meter_1 (Union[Unset, RInstanceWithUnit]):
        trip_meter_2 (Union[Unset, RInstanceWithUnit]):
        average_fuel_consumption (Union[Unset, RInstanceWithUnit]):
    """

    average_speed: Union[Unset, "RInstanceWithUnit"] = UNSET
    distance_to_empty: Union[Unset, "RInstanceWithUnit"] = UNSET
    trip_meter_1: Union[Unset, "RInstanceWithUnit"] = UNSET
    trip_meter_2: Union[Unset, "RInstanceWithUnit"] = UNSET
    average_fuel_consumption: Union[Unset, "RInstanceWithUnit"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        average_speed: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.average_speed, Unset):
            average_speed = self.average_speed.to_dict()

        distance_to_empty: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.distance_to_empty, Unset):
            distance_to_empty = self.distance_to_empty.to_dict()

        trip_meter_1: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trip_meter_1, Unset):
            trip_meter_1 = self.trip_meter_1.to_dict()

        trip_meter_2: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trip_meter_2, Unset):
            trip_meter_2 = self.trip_meter_2.to_dict()

        average_fuel_consumption: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.average_fuel_consumption, Unset):
            average_fuel_consumption = self.average_fuel_consumption.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if average_speed is not UNSET:
            field_dict["averageSpeed"] = average_speed
        if distance_to_empty is not UNSET:
            field_dict["distanceToEmpty"] = distance_to_empty
        if trip_meter_1 is not UNSET:
            field_dict["tripMeter1"] = trip_meter_1
        if trip_meter_2 is not UNSET:
            field_dict["tripMeter2"] = trip_meter_2
        if average_fuel_consumption is not UNSET:
            field_dict["averageFuelConsumption"] = average_fuel_consumption

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.r_instance_with_unit import RInstanceWithUnit

        d = src_dict.copy()
        _average_speed = d.pop("averageSpeed", UNSET)
        average_speed: Union[Unset, RInstanceWithUnit]
        if isinstance(_average_speed, Unset):
            average_speed = UNSET
        else:
            average_speed = RInstanceWithUnit.from_dict(_average_speed)

        _distance_to_empty = d.pop("distanceToEmpty", UNSET)
        distance_to_empty: Union[Unset, RInstanceWithUnit]
        if isinstance(_distance_to_empty, Unset):
            distance_to_empty = UNSET
        else:
            distance_to_empty = RInstanceWithUnit.from_dict(_distance_to_empty)

        _trip_meter_1 = d.pop("tripMeter1", UNSET)
        trip_meter_1: Union[Unset, RInstanceWithUnit]
        if isinstance(_trip_meter_1, Unset):
            trip_meter_1 = UNSET
        else:
            trip_meter_1 = RInstanceWithUnit.from_dict(_trip_meter_1)

        _trip_meter_2 = d.pop("tripMeter2", UNSET)
        trip_meter_2: Union[Unset, RInstanceWithUnit]
        if isinstance(_trip_meter_2, Unset):
            trip_meter_2 = UNSET
        else:
            trip_meter_2 = RInstanceWithUnit.from_dict(_trip_meter_2)

        _average_fuel_consumption = d.pop("averageFuelConsumption", UNSET)
        average_fuel_consumption: Union[Unset, RInstanceWithUnit]
        if isinstance(_average_fuel_consumption, Unset):
            average_fuel_consumption = UNSET
        else:
            average_fuel_consumption = RInstanceWithUnit.from_dict(_average_fuel_consumption)

        statistic_vals = cls(
            average_speed=average_speed,
            distance_to_empty=distance_to_empty,
            trip_meter_1=trip_meter_1,
            trip_meter_2=trip_meter_2,
            average_fuel_consumption=average_fuel_consumption,
        )

        statistic_vals.additional_properties = d
        return statistic_vals

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.description_texts import DescriptionTexts
    from ..models.vehicle_images import VehicleImages


T = TypeVar("T", bound="VehicleMetadata")


@attr.s(auto_attribs=True)
class VehicleMetadata:
    """
    Attributes:
        model_year (Union[Unset, str]):
        vin (Union[Unset, str]):
        external_colour (Union[Unset, str]):
        images (Union[Unset, VehicleImages]):
        descriptions (Union[Unset, DescriptionTexts]):
    """

    model_year: Union[Unset, str] = UNSET
    vin: Union[Unset, str] = UNSET
    external_colour: Union[Unset, str] = UNSET
    images: Union[Unset, "VehicleImages"] = UNSET
    descriptions: Union[Unset, "DescriptionTexts"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        model_year = self.model_year
        vin = self.vin
        external_colour = self.external_colour
        images: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.images, Unset):
            images = self.images.to_dict()

        descriptions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.descriptions, Unset):
            descriptions = self.descriptions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if model_year is not UNSET:
            field_dict["modelYear"] = model_year
        if vin is not UNSET:
            field_dict["vin"] = vin
        if external_colour is not UNSET:
            field_dict["externalColour"] = external_colour
        if images is not UNSET:
            field_dict["images"] = images
        if descriptions is not UNSET:
            field_dict["descriptions"] = descriptions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.description_texts import DescriptionTexts
        from ..models.vehicle_images import VehicleImages

        d = src_dict.copy()
        model_year = d.pop("modelYear", UNSET)

        vin = d.pop("vin", UNSET)

        external_colour = d.pop("externalColour", UNSET)

        _images = d.pop("images", UNSET)
        images: Union[Unset, VehicleImages]
        if isinstance(_images, Unset):
            images = UNSET
        else:
            images = VehicleImages.from_dict(_images)

        _descriptions = d.pop("descriptions", UNSET)
        descriptions: Union[Unset, DescriptionTexts]
        if isinstance(_descriptions, Unset):
            descriptions = UNSET
        else:
            descriptions = DescriptionTexts.from_dict(_descriptions)

        vehicle_metadata = cls(
            model_year=model_year,
            vin=vin,
            external_colour=external_colour,
            images=images,
            descriptions=descriptions,
        )

        vehicle_metadata.additional_properties = d
        return vehicle_metadata

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...models.engine_start_request import EngineStartRequest
from ...models.error_response import ErrorResponse
from ...models.invocation_result import InvocationResult
from ...types import UNSET, Response, Unset


def _get_kwargs(
    vin: str,
    *,
    client: Client,
    json_body: EngineStartRequest,
    vcc_api_operation_id: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/vehicles/{vin}/commands/engine-start".format(client.base_url, vin=vin)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(vcc_api_operation_id, Unset):
        headers["vcc-api-operationId"] = vcc_api_operation_id

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "json": json_json_body,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Union[ErrorResponse, InvocationResult]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = InvocationResult.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ErrorResponse.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ErrorResponse.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ErrorResponse.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = ErrorResponse.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.UNSUPPORTED_MEDIA_TYPE:
        response_415 = ErrorResponse.from_dict(response.json())

        return response_415
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = ErrorResponse.from_dict(response.json())

        return response_422
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ErrorResponse.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.SERVICE_UNAVAILABLE:
        response_503 = ErrorResponse.from_dict(response.json())

        return response_503
    if response.status_code == HTTPStatus.GATEWAY_TIMEOUT:
        response_504 = ErrorResponse.from_dict(response.json())

        return response_504
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Union[ErrorResponse, InvocationResult]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    vin: str,
    *,
    client: Client,
    json_body: EngineStartRequest,
    vcc_api_operation_id: Union[Unset, str] = UNSET,
) -> Response[Union[ErrorResponse, InvocationResult]]:
    """Start Engine

     Used to send a engine start command to the vehicle.

    Args:
        vin (str):
        vcc_api_operation_id (Union[Unset, str]):
        json_body (EngineStartRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ErrorResponse, InvocationResult]]
    """

    kwargs = _get_kwargs(
        vin=vin,
        client=client,
        json_body=json_body,
        vcc_api_operation_id=vcc_api_operation_id,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    vin: str,
    *,
    client: Client,
    json_body: EngineStartRequest,
    vcc_api_operation_id: Union[Unset, str] = UNSET,
) -> Optional[Union[ErrorResponse, InvocationResult]]:
    """Start Engine

     Used to send a engine start command to the vehicle.

    Args:
        vin (str):
        vcc_api_operation_id (Union[Unset, str]):
        json_body (EngineStartRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ErrorResponse, InvocationResult]
    """

    return sync_detailed(
        vin=vin,
        client=client,
        json_body=json_body,
        vcc_api_operation_id=vcc_api_operation_id,
    ).parsed


async def asyncio_detailed(
    vin: str,
    *,
    client: Client,
    json_body: EngineStartRequest,
    vcc_api_operation_id: Union[Unset, str] = UNSET,
) -> Response[Union[ErrorResponse, InvocationResult]]:
    """Start Engine

     Used to send a engine start command to the vehicle.

    Args:
        vin (str):
        vcc_api_operation_id (Union[Unset, str]):
        json_body (EngineStartRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ErrorResponse, InvocationResult]]
    """

    kwargs = _get_kwargs(
        vin=vin,
        client=client,
        json_body=json_body,
        vcc_api_operation_id=vcc_api_operation_id,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    vin: str,
    *,
    client: Client,
    json_body: EngineStartRequest,
    vcc_api_operation_id: Union[Unset, str] = UNSET,
) -> Optional[Union[ErrorResponse, InvocationResult]]:
    """Start Engine

     Used to send a engine start command to the vehicle.

    Args:
        vin (str):
        vcc_api_operation_id (Union[Unset, str]):
        json_body (EngineStartRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[ErrorResponse, InvocationResult]
    """

    return (
        await asyncio_detailed(
            vin=vin,
            client=client,
            json_body=json_body,
            vcc_api_operation_id=vcc_api_operation_id,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.invocation_result_invoke_status import InvocationResultInvokeStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="InvocationResult")


@attr.s(auto_attribs=True)
class InvocationResult:
    """
    Attributes:
        vin (Union[Unset, str]):
        status (Union[Unset, str]):
        status_code (Union[Unset, int]):
        operation_id (Union[Unset, str]):
        invoke_status (Union[Unset, InvocationResultInvokeStatus]):
        message (Union[Unset, str]):
    """

    vin: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    status_code: Union[Unset, int] = UNSET
    operation_id: Union[Unset, str] = UNSET
    invoke_status: Union[Unset, InvocationResultInvokeStatus] = UNSET
    message: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        vin = self.vin
        status = self.status
        status_code = self.status_code
        operation_id = self.operation_id
        invoke_status: Union[Unset, str] = UNSET
        if not isinstance(self.invoke_status, Unset):
            invoke_status = self.invoke_status.value

        message = self.message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if vin is not UNSET:
            field_dict["vin"] = vin
        if status is not UNSET:
            field_dict["status"] = status
        if status_code is not UNSET:
            field_dict["statusCode"] = status_code
        if operation_id is not UNSET:
            field_dict["operationId"] = operation_id
        if invoke_status is not UNSET:
            field_dict["invokeStatus"] = invoke_status
        if message is not UNSET:
            field_dict["message"] = message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        vin = d.pop("vin", UNSET)

        status = d.pop("status", UNSET)

        status_code = d.pop("statusCode", UNSET)

        operation_id = d.pop("operationId", UNSET)

        _invoke_status = d.pop("invokeStatus", UNSET)
        invoke_status: Union[Unset, InvocationResultInvokeStatus]
        if isinstance(_invoke_status, Unset):
            invoke_status = UNSET
        else:
            invoke_status = InvocationResultInvokeStatus(_invoke_status)

        message = d.pop("message", UNSET)

        invocation_result = cls(
            vin=vin,
            status=status,
            status_code=status_code,
            operation_id=operation_id,
            invoke_status=invoke_status,
            message=message,
        )

        invocation_result.additional_properties = d
        return invocation_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.command_list_item_command import CommandListItemCommand
from ..types import UNSET, Unset

T = TypeVar("T", bound="CommandListItem")


@attr.s(auto_attribs=True)
class CommandListItem:
    """
    Attributes:
        command (Union[Unset, CommandListItemCommand]):
        href (Union[Unset, str]):
    """

    command: Union[Unset, CommandListItemCommand] = UNSET
    href: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        command: Union[Unset, str] = UNSET
        if not isinstance(self.command, Unset):
            command = self.command.value

        href = self.href

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if command is not UNSET:
            field_dict["command"] = command
        if href is not UNSET:
            field_dict["href"] = href

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _command = d.pop("command", UNSET)
        command: Union[Unset, CommandListItemCommand]
        if isinstance(_command, Unset):
            command = UNSET
        else:
            command = CommandListItemCommand(_command)

        href = d.pop("href", UNSET)

        command_list_item = cls(
            command=command,
            href=href,
        )

        command_list_item.additional_properties = d
        return command_list_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.r_instance import RInstance


T = TypeVar("T", bound="EngineStatus")


@attr.s(auto_attribs=True)
class EngineStatus:
    """
    Attributes:
        engine_running (Union[Unset, RInstance]):
    """

    engine_running: Union[Unset, "RInstance"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        engine_running: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.engine_running, Unset):
            engine_running = self.engine_running.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if engine_running is not UNSET:
            field_dict["engineRunning"] = engine_running

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.r_instance import RInstance

        d = src_dict.copy()
        _engine_running = d.pop("engineRunning", UNSET)
        engine_running: Union[Unset, RInstance]
        if isinstance(_engine_running, Unset):
            engine_running = UNSET
        else:
            engine_running = RInstance.from_dict(_engine_running)

        engine_status = cls(
            engine_running=engine_running,
        )

        engine_status.additional_properties = d
        return engine_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.r_instance import RInstance


T = TypeVar("T", bound="DoorAndLockStatus")


@attr.s(auto_attribs=True)
class DoorAndLockStatus:
    """
    Attributes:
        car_locked (Union[Unset, RInstance]):
        rear_right (Union[Unset, RInstance]):
        front_right (Union[Unset, RInstance]):
        front_left (Union[Unset, RInstance]):
        rear_left (Union[Unset, RInstance]):
        cap (Union[Unset, RInstance]):
        hood (Union[Unset, RInstance]):
        tail_gate (Union[Unset, RInstance]):
    """

    car_locked: Union[Unset, "RInstance"] = UNSET
    rear_right: Union[Unset, "RInstance"] = UNSET
    front_right: Union[Unset, "RInstance"] = UNSET
    front_left: Union[Unset, "RInstance"] = UNSET
    rear_left: Union[Unset, "RInstance"] = UNSET
    cap: Union[Unset, "RInstance"] = UNSET
    hood: Union[Unset, "RInstance"] = UNSET
    tail_gate: Union[Unset, "RInstance"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        car_locked: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.car_locked, Unset):
            car_locked = self.car_locked.to_dict()

        rear_right: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.rear_right, Unset):
            rear_right = self.rear_right.to_dict()

        front_right: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.front_right, Unset):
            front_right = self.front_right.to_dict()

        front_left: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.front_left, Unset):
            front_left = self.front_left.to_dict()

        rear_left: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.rear_left, Unset):
            rear_left = self.rear_left.to_dict()

        cap: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cap, Unset):
            cap = self.cap.to_dict()

        hood: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.hood, Unset):
            hood = self.hood.to_dict()

        tail_gate: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tail_gate, Unset):
            tail_gate = self.tail_gate.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if car_locked is not UNSET:
            field_dict["carLocked"] = car_locked
        if rear_right is not UNSET:
            field_dict["rearRight"] = rear_right
        if front_right is not UNSET:
            field_dict["frontRight"] = front_right
        if front_left is not UNSET:
            field_dict["frontLeft"] = front_left
        if rear_left is not UNSET:
            field_dict["rearLeft"] = rear_left
        if cap is not UNSET:
            field_dict["cap"] = cap
        if hood is not UNSET:
            field_dict["hood"] = hood
        if tail_gate is not UNSET:
            field_dict["tailGate"] = tail_gate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.r_instance import RInstance

        d = src_dict.copy()
        _car_locked = d.pop("carLocked", UNSET)
        car_locked: Union[Unset, RInstance]
        if isinstance(_car_locked, Unset):
            car_locked = UNSET
        else:
            car_locked = RInstance.from_dict(_car_locked)

        _rear_right = d.pop("rearRight", UNSET)
        rear_right: Union[Unset, RInstance]
        if isinstance(_rear_right, Unset):
            rear_right = UNSET
        else:
            rear_right = RInstance.from_dict(_rear_right)

        _front_right = d.pop("frontRight", UNSET)
        front_right: Union[Unset, RInstance]
        if isinstance(_front_right, Unset):
            front_right = UNSET
        else:
            front_right = RInstance.from_dict(_front_right)

        _front_left = d.pop("frontLeft", UNSET)
        front_left: Union[Unset, RInstance]
        if isinstance(_front_left, Unset):
            front_left = UNSET
        else:
            front_left = RInstance.from_dict(_front_left)

        _rear_left = d.pop("rearLeft", UNSET)
        rear_left: Union[Unset, RInstance]
        if isinstance(_rear_left, Unset):
            rear_left = UNSET
        else:
            rear_left = RInstance.from_dict(_rear_left)

        _cap = d.pop("cap", UNSET)
        cap: Union[Unset, RInstance]
        if isinstance(_cap, Unset):
            cap = UNSET
        else:
            cap = RInstance.from_dict(_cap)

        _hood = d.pop("hood", UNSET)
        hood: Union[Unset, RInstance]
        if isinstance(_hood, Unset):
            hood = UNSET
        else:
            hood = RInstance.from_dict(_hood)

        _tail_gate = d.pop("tailGate", UNSET)
        tail_gate: Union[Unset, RInstance]
        if isinstance(_tail_gate, Unset):
            tail_gate = UNSET
        else:
            tail_gate = RInstance.from_dict(_tail_gate)

        door_and_lock_status = cls(
            car_locked=car_locked,
            rear_right=rear_right,
            front_right=front_right,
            front_left=front_left,
            rear_left=rear_left,
            cap=cap,
            hood=hood,
            tail_gate=tail_gate,
        )

        door_and_lock_status.additional_properties = d
        return door_and_lock_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.r_instance import RInstance


T = TypeVar("T", bound="WindowStatus")


@attr.s(auto_attribs=True)
class WindowStatus:
    """
    Attributes:
        front_left (Union[Unset, RInstance]):
        front_right (Union[Unset, RInstance]):
        rear_left (Union[Unset, RInstance]):
        rear_right (Union[Unset, RInstance]):
    """

    front_left: Union[Unset, "RInstance"] = UNSET
    front_right: Union[Unset, "RInstance"] = UNSET
    rear_left: Union[Unset, "RInstance"] = UNSET
    rear_right: Union[Unset, "RInstance"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        front_left: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.front_left, Unset):
            front_left = self.front_left.to_dict()

        front_right: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.front_right, Unset):
            front_right = self.front_right.to_dict()

        rear_left: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.rear_left, Unset):
            rear_left = self.rear_left.to_dict()

        rear_right: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.rear_right, Unset):
            rear_right = self.rear_right.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if front_left is not UNSET:
            field_dict["frontLeft"] = front_left
        if front_right is not UNSET:
            field_dict["frontRight"] = front_right
        if rear_left is not UNSET:
            field_dict["rearLeft"] = rear_left
        if rear_right is not UNSET:
            field_dict["rearRight"] = rear_right

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.r_instance import RInstance

        d = src_dict.copy()
        _front_left = d.pop("frontLeft", UNSET)
        front_left: Union[Unset, RInstance]
        if isinstance(_front_left, Unset):
            front_left = UNSET
        else:
            front_left = RInstance.from_dict(_front_left)

        _front_right = d.pop("frontRight", UNSET)
        front_right: Union[Unset, RInstance]
        if isinstance(_front_right, Unset):
            front_right = UNSET
        else:
            front_right = RInstance.from_dict(_front_right)

        _rear_left = d.pop("rearLeft", UNSET)
        rear_left: Union[Unset, RInstance]
        if isinstance(_rear_left, Unset):
            rear_left = UNSET
        else:
            rear_left = RInstance.from_dict(_rear_left)

        _rear_right = d.pop("rearRight", UNSET)
        rear_right: Union[Unset, RInstance]
        if isinstance(_rear_right, Unset):
            rear_right = UNSET
        else:
            rear_right = RInstance.from_dict(_rear_right)

        window_status = cls(
            front_left=front_left,
            front_right=front_right,
            rear_left=rear_left,
            rear_right=rear_right,
        )

        window_status.additional_properties = d
        return window_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.statistic_vals import StatisticVals


T = TypeVar("T", bound="StatisticResponse")


@attr.s(auto_attribs=True)
class StatisticResponse:
    """
    Attributes:
        status (Union[Unset, int]):
        operation_id (Union[Unset, str]):
        data (Union[Unset, StatisticVals]):
    """

    status: Union[Unset, int] = UNSET
    operation_id: Union[Unset, str] = UNSET
    data: Union[Unset, "StatisticVals"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status = self.status
        operation_id = self.operation_id
        data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data, Unset):
            data = self.data.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status
        if operation_id is not UNSET:
            field_dict["operationId"] = operation_id
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.statistic_vals import StatisticVals

        d = src_dict.copy()
        status = d.pop("status", UNSET)

        operation_id = d.pop("operationId", UNSET)

        _data = d.pop("data", UNSET)
        data: Union[Unset, StatisticVals]
        if isinstance(_data, Unset):
            data = UNSET
        else:
            data = StatisticVals.from_dict(_data)

        statistic_response = cls(
            status=status,
            operation_id=operation_id,
            data=data,
        )

        statistic_response.additional_properties = d
        return statistic_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.r_instance import RInstance
    from ..models.r_instance_with_unit import RInstanceWithUnit


T = TypeVar("T", bound="DiagnosticVals")


@attr.s(auto_attribs=True)
class DiagnosticVals:
    """
    Attributes:
        engine_hours_to_service (Union[Unset, RInstance]):
        km_to_service (Union[Unset, RInstanceWithUnit]):
        main_battery_status (Union[Unset, RInstance]):
        months_to_service (Union[Unset, RInstance]):
        service_type (Union[Unset, RInstance]):
        service_status (Union[Unset, RInstance]):
        service_trigger (Union[Unset, RInstance]):
        backup_battery_remaining (Union[Unset, RInstance]):
        washer_fluid_level (Union[Unset, RInstance]):
    """

    engine_hours_to_service: Union[Unset, "RInstance"] = UNSET
    km_to_service: Union[Unset, "RInstanceWithUnit"] = UNSET
    main_battery_status: Union[Unset, "RInstance"] = UNSET
    months_to_service: Union[Unset, "RInstance"] = UNSET
    service_type: Union[Unset, "RInstance"] = UNSET
    service_status: Union[Unset, "RInstance"] = UNSET
    service_trigger: Union[Unset, "RInstance"] = UNSET
    backup_battery_remaining: Union[Unset, "RInstance"] = UNSET
    washer_fluid_level: Union[Unset, "RInstance"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        engine_hours_to_service: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.engine_hours_to_service, Unset):
            engine_hours_to_service = self.engine_hours_to_service.to_dict()

        km_to_service: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.km_to_service, Unset):
            km_to_service = self.km_to_service.to_dict()

        main_battery_status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.main_battery_status, Unset):
            main_battery_status = self.main_battery_status.to_dict()

        months_to_service: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.months_to_service, Unset):
            months_to_service = self.months_to_service.to_dict()

        service_type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.service_type, Unset):
            service_type = self.service_type.to_dict()

        service_status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.service_status, Unset):
            service_status = self.service_status.to_dict()

        service_trigger: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.service_trigger, Unset):
            service_trigger = self.service_trigger.to_dict()

        backup_battery_remaining: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.backup_battery_remaining, Unset):
            backup_battery_remaining = self.backup_battery_remaining.to_dict()

        washer_fluid_level: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.washer_fluid_level, Unset):
            washer_fluid_level = self.washer_fluid_level.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if engine_hours_to_service is not UNSET:
            field_dict["engineHoursToService"] = engine_hours_to_service
        if km_to_service is not UNSET:
            field_dict["kmToService"] = km_to_service
        if main_battery_status is not UNSET:
            field_dict["mainBatteryStatus"] = main_battery_status
        if months_to_service is not UNSET:
            field_dict["monthsToService"] = months_to_service
        if service_type is not UNSET:
            field_dict["serviceType"] = service_type
        if service_status is not UNSET:
            field_dict["serviceStatus"] = service_status
        if service_trigger is not UNSET:
            field_dict["serviceTrigger"] = service_trigger
        if backup_battery_remaining is not UNSET:
            field_dict["backupBatteryRemaining"] = backup_battery_remaining
        if washer_fluid_level is not UNSET:
            field_dict["washerFluidLevel"] = washer_fluid_level

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.r_instance import RInstance
        from ..models.r_instance_with_unit import RInstanceWithUnit

        d = src_dict.copy()
        _engine_hours_to_service = d.pop("engineHoursToService", UNSET)
        engine_hours_to_service: Union[Unset, RInstance]
        if isinstance(_engine_hours_to_service, Unset):
            engine_hours_to_service = UNSET
        else:
            engine_hours_to_service = RInstance.from_dict(_engine_hours_to_service)

        _km_to_service = d.pop("kmToService", UNSET)
        km_to_service: Union[Unset, RInstanceWithUnit]
        if isinstance(_km_to_service, Unset):
            km_to_service = UNSET
        else:
            km_to_service = RInstanceWithUnit.from_dict(_km_to_service)

        _main_battery_status = d.pop("mainBatteryStatus", UNSET)
        main_battery_status: Union[Unset, RInstance]
        if isinstance(_main_battery_status, Unset):
            main_battery_status = UNSET
        else:
            main_battery_status = RInstance.from_dict(_main_battery_status)

        _months_to_service = d.pop("monthsToService", UNSET)
        months_to_service: Union[Unset, RInstance]
        if isinstance(_months_to_service, Unset):
            months_to_service = UNSET
        else:
            months_to_service = RInstance.from_dict(_months_to_service)

        _service_type = d.pop("serviceType", UNSET)
        service_type: Union[Unset, RInstance]
        if isinstance(_service_type, Unset):
            service_type = UNSET
        else:
            service_type = RInstance.from_dict(_service_type)

        _service_status = d.pop("serviceStatus", UNSET)
        service_status: Union[Unset, RInstance]
        if isinstance(_service_status, Unset):
            service_status = UNSET
        else:
            service_status = RInstance.from_dict(_service_status)

        _service_trigger = d.pop("serviceTrigger", UNSET)
        service_trigger: Union[Unset, RInstance]
        if isinstance(_service_trigger, Unset):
            service_trigger = UNSET
        else:
            service_trigger = RInstance.from_dict(_service_trigger)

        _backup_battery_remaining = d.pop("backupBatteryRemaining", UNSET)
        backup_battery_remaining: Union[Unset, RInstance]
        if isinstance(_backup_battery_remaining, Unset):
            backup_battery_remaining = UNSET
        else:
            backup_battery_remaining = RInstance.from_dict(_backup_battery_remaining)

        _washer_fluid_level = d.pop("washerFluidLevel", UNSET)
        washer_fluid_level: Union[Unset, RInstance]
        if isinstance(_washer_fluid_level, Unset):
            washer_fluid_level = UNSET
        else:
            washer_fluid_level = RInstance.from_dict(_washer_fluid_level)

        diagnostic_vals = cls(
            engine_hours_to_service=engine_hours_to_service,
            km_to_service=km_to_service,
            main_battery_status=main_battery_status,
            months_to_service=months_to_service,
            service_type=service_type,
            service_status=service_status,
            service_trigger=service_trigger,
            backup_battery_remaining=backup_battery_remaining,
            washer_fluid_level=washer_fluid_level,
        )

        diagnostic_vals.additional_properties = d
        return diagnostic_vals

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="RInstanceWithUnit")


@attr.s(auto_attribs=True)
class RInstanceWithUnit:
    """
    Attributes:
        value (Union[Unset, str]):
        timestamp (Union[Unset, datetime.datetime]):
        unit (Union[Unset, str]):
    """

    value: Union[Unset, str] = UNSET
    timestamp: Union[Unset, datetime.datetime] = UNSET
    unit: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        value = self.value
        timestamp: Union[Unset, str] = UNSET
        if not isinstance(self.timestamp, Unset):
            timestamp = self.timestamp.isoformat()

        unit = self.unit

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if value is not UNSET:
            field_dict["value"] = value
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        value = d.pop("value", UNSET)

        _timestamp = d.pop("timestamp", UNSET)
        timestamp: Union[Unset, datetime.datetime]
        if isinstance(_timestamp, Unset):
            timestamp = UNSET
        else:
            timestamp = isoparse(_timestamp)

        unit = d.pop("unit", UNSET)

        r_instance_with_unit = cls(
            value=value,
            timestamp=timestamp,
            unit=unit,
        )

        r_instance_with_unit.additional_properties = d
        return r_instance_with_unit

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class UnlockResultDetailV2InvokeStatus(str, Enum):
    CAR_ERROR = "CAR_ERROR"
    CAR_IN_SLEEP_MODE = "CAR_IN_SLEEP_MODE"
    CAR_TIMEOUT = "CAR_TIMEOUT"
    COMPLETED = "COMPLETED"
    CONNECTION_FAILURE = "CONNECTION_FAILURE"
    DELIVERED = "DELIVERED"
    DELIVERY_TIMEOUT = "DELIVERY_TIMEOUT"
    EXPIRED = "EXPIRED"
    NOT_ALLOWED_PRIVACY_ENABLED = "NOT_ALLOWED_PRIVACY_ENABLED"
    NOT_ALLOWED_WRONG_USAGE_MODE = "NOT_ALLOWED_WRONG_USAGE_MODE"
    NOT_SUPPORTED = "NOT_SUPPORTED"
    REJECTED = "REJECTED"
    RUNNING = "RUNNING"
    SENT = "SENT"
    SUCCESS = "SUCCESS"
    TIMEOUT = "TIMEOUT"
    UNABLE_TO_LOCK_DOOR_OPEN = "UNABLE_TO_LOCK_DOOR_OPEN"
    UNKNOWN = "UNKNOWN"
    UNLOCK_TIME_FRAME_PASSED = "UNLOCK_TIME_FRAME_PASSED"
    VEHICLE_IN_SLEEP = "VEHICLE_IN_SLEEP"
    WAITING = "WAITING"

    def __str__(self) -> str:
        return str(self.value)



""" Contains all the data models used in inputs/outputs """

from .battery_charge import BatteryCharge
from .battery_charge_level_response import BatteryChargeLevelResponse
from .brake_status import BrakeStatus
from .brake_status_response import BrakeStatusResponse
from .command_accessibility import CommandAccessibility
from .command_accessibility_response import CommandAccessibilityResponse
from .command_list_item import CommandListItem
from .command_list_item_command import CommandListItemCommand
from .command_list_response import CommandListResponse
from .description_texts import DescriptionTexts
from .diagnostic_response import DiagnosticResponse
from .diagnostic_vals import DiagnosticVals
from .door_and_lock_status import DoorAndLockStatus
from .door_status_response import DoorStatusResponse
from .engine_diagnostic_response import EngineDiagnosticResponse
from .engine_diagnostic_vals import EngineDiagnosticVals
from .engine_start_request import EngineStartRequest
from .engine_status import EngineStatus
from .engine_status_response import EngineStatusResponse
from .environment_response import EnvironmentResponse
from .environment_vals import EnvironmentVals
from .error import Error
from .error_response import ErrorResponse
from .fuel import Fuel
from .fuel_amount_response import FuelAmountResponse
from .invocation_result import InvocationResult
from .invocation_result_invoke_status import InvocationResultInvokeStatus
from .odometer import Odometer
from .odometer_response import OdometerResponse
from .pagination import Pagination
from .r_instance import RInstance
from .r_instance_with_unit import RInstanceWithUnit
from .statistic_response import StatisticResponse
from .statistic_vals import StatisticVals
from .tyre_pressure import TyrePressure
from .tyre_pressure_response import TyrePressureResponse
from .unlock_response_v2 import UnlockResponseV2
from .unlock_result_detail_v2 import UnlockResultDetailV2
from .unlock_result_detail_v2_invoke_status import UnlockResultDetailV2InvokeStatus
from .vehicle_detail_response import VehicleDetailResponse
from .vehicle_images import VehicleImages
from .vehicle_list_item import VehicleListItem
from .vehicle_list_response import VehicleListResponse
from .vehicle_metadata import VehicleMetadata
from .warning_vals import WarningVals
from .warnings_response import WarningsResponse
from .window_status import WindowStatus
from .window_status_response import WindowStatusResponse

__all__ = (
    "BatteryCharge",
    "BatteryChargeLevelResponse",
    "BrakeStatus",
    "BrakeStatusResponse",
    "CommandAccessibility",
    "CommandAccessibilityResponse",
    "CommandListItem",
    "CommandListItemCommand",
    "CommandListResponse",
    "DescriptionTexts",
    "DiagnosticResponse",
    "DiagnosticVals",
    "DoorAndLockStatus",
    "DoorStatusResponse",
    "EngineDiagnosticResponse",
    "EngineDiagnosticVals",
    "EngineStartRequest",
    "EngineStatus",
    "EngineStatusResponse",
    "EnvironmentResponse",
    "EnvironmentVals",
    "Error",
    "ErrorResponse",
    "Fuel",
    "FuelAmountResponse",
    "InvocationResult",
    "InvocationResultInvokeStatus",
    "Odometer",
    "OdometerResponse",
    "Pagination",
    "RInstance",
    "RInstanceWithUnit",
    "StatisticResponse",
    "StatisticVals",
    "TyrePressure",
    "TyrePressureResponse",
    "UnlockResponseV2",
    "UnlockResultDetailV2",
    "UnlockResultDetailV2InvokeStatus",
    "VehicleDetailResponse",
    "VehicleImages",
    "VehicleListItem",
    "VehicleListResponse",
    "VehicleMetadata",
    "WarningsResponse",
    "WarningVals",
    "WindowStatus",
    "WindowStatusResponse",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.r_instance import RInstance
    from ..models.r_instance_with_unit import RInstanceWithUnit


T = TypeVar("T", bound="EngineDiagnosticVals")


@attr.s(auto_attribs=True)
class EngineDiagnosticVals:
    """
    Attributes:
        engine_coolant_level (Union[Unset, RInstance]):
        oil_level (Union[Unset, RInstance]):
        oil_pressure (Union[Unset, RInstance]):
        engine_coolant_temp (Union[Unset, RInstanceWithUnit]):
    """

    engine_coolant_level: Union[Unset, "RInstance"] = UNSET
    oil_level: Union[Unset, "RInstance"] = UNSET
    oil_pressure: Union[Unset, "RInstance"] = UNSET
    engine_coolant_temp: Union[Unset, "RInstanceWithUnit"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        engine_coolant_level: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.engine_coolant_level, Unset):
            engine_coolant_level = self.engine_coolant_level.to_dict()

        oil_level: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.oil_level, Unset):
            oil_level = self.oil_level.to_dict()

        oil_pressure: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.oil_pressure, Unset):
            oil_pressure = self.oil_pressure.to_dict()

        engine_coolant_temp: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.engine_coolant_temp, Unset):
            engine_coolant_temp = self.engine_coolant_temp.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if engine_coolant_level is not UNSET:
            field_dict["engineCoolantLevel"] = engine_coolant_level
        if oil_level is not UNSET:
            field_dict["oilLevel"] = oil_level
        if oil_pressure is not UNSET:
            field_dict["oilPressure"] = oil_pressure
        if engine_coolant_temp is not UNSET:
            field_dict["engineCoolantTemp"] = engine_coolant_temp

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.r_instance import RInstance
        from ..models.r_instance_with_unit import RInstanceWithUnit

        d = src_dict.copy()
        _engine_coolant_level = d.pop("engineCoolantLevel", UNSET)
        engine_coolant_level: Union[Unset, RInstance]
        if isinstance(_engine_coolant_level, Unset):
            engine_coolant_level = UNSET
        else:
            engine_coolant_level = RInstance.from_dict(_engine_coolant_level)

        _oil_level = d.pop("oilLevel", UNSET)
        oil_level: Union[Unset, RInstance]
        if isinstance(_oil_level, Unset):
            oil_level = UNSET
        else:
            oil_level = RInstance.from_dict(_oil_level)

        _oil_pressure = d.pop("oilPressure", UNSET)
        oil_pressure: Union[Unset, RInstance]
        if isinstance(_oil_pressure, Unset):
            oil_pressure = UNSET
        else:
            oil_pressure = RInstance.from_dict(_oil_pressure)

        _engine_coolant_temp = d.pop("engineCoolantTemp", UNSET)
        engine_coolant_temp: Union[Unset, RInstanceWithUnit]
        if isinstance(_engine_coolant_temp, Unset):
            engine_coolant_temp = UNSET
        else:
            engine_coolant_temp = RInstanceWithUnit.from_dict(_engine_coolant_temp)

        engine_diagnostic_vals = cls(
            engine_coolant_level=engine_coolant_level,
            oil_level=oil_level,
            oil_pressure=oil_pressure,
            engine_coolant_temp=engine_coolant_temp,
        )

        engine_diagnostic_vals.additional_properties = d
        return engine_diagnostic_vals

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="VehicleImages")


@attr.s(auto_attribs=True)
class VehicleImages:
    """
    Attributes:
        exterior_default_url (Union[Unset, str]):
        interior_default_url (Union[Unset, str]):
    """

    exterior_default_url: Union[Unset, str] = UNSET
    interior_default_url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        exterior_default_url = self.exterior_default_url
        interior_default_url = self.interior_default_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if exterior_default_url is not UNSET:
            field_dict["exteriorDefaultUrl"] = exterior_default_url
        if interior_default_url is not UNSET:
            field_dict["interiorDefaultUrl"] = interior_default_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        exterior_default_url = d.pop("exteriorDefaultUrl", UNSET)

        interior_default_url = d.pop("interiorDefaultUrl", UNSET)

        vehicle_images = cls(
            exterior_default_url=exterior_default_url,
            interior_default_url=interior_default_url,
        )

        vehicle_images.additional_properties = d
        return vehicle_images

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="CommandAccessibility")


@attr.s(auto_attribs=True)
class CommandAccessibility:
    """
    Attributes:
        accessible (Union[Unset, bool]):
        accessible_until (Union[Unset, datetime.datetime]):
    """

    accessible: Union[Unset, bool] = UNSET
    accessible_until: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        accessible = self.accessible
        accessible_until: Union[Unset, str] = UNSET
        if not isinstance(self.accessible_until, Unset):
            accessible_until = self.accessible_until.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if accessible is not UNSET:
            field_dict["accessible"] = accessible
        if accessible_until is not UNSET:
            field_dict["accessibleUntil"] = accessible_until

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        accessible = d.pop("accessible", UNSET)

        _accessible_until = d.pop("accessibleUntil", UNSET)
        accessible_until: Union[Unset, datetime.datetime]
        if isinstance(_accessible_until, Unset):
            accessible_until = UNSET
        else:
            accessible_until = isoparse(_accessible_until)

        command_accessibility = cls(
            accessible=accessible,
            accessible_until=accessible_until,
        )

        command_accessibility.additional_properties = d
        return command_accessibility

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
